unit uReportUI;
{$INCLUDE 'Plugin.inc'}

interface

uses
  SysUtils, Windows, Classes, StrUtils, Generics.Collections, Generics.Defaults,
  {$INCLUDE 'DelphiFarUnits.inc'} ,
  uVirusTotal;

type
  TReportDialog = class(TSearchDialog)
  public
    type TReloadReportEvent = procedure(Sender: TReportDialog; var NewReport: IVirusTotalReport) of object;
  private
    fFileName: string;
    fReport: IVirusTotalReport;
    fButtonRescan: TFarDialog.TButton;
    fButtonEditor: TFarDialog.TButton;
    fLabelTotals: TFarDialog.TLabel;
    fOnReloadReport: TReloadReportEvent;
    fData: TStream;
  protected
    procedure CreateControls; override;
    procedure Clear; override;
    procedure SetSize(Width, Height: integer); override;
    function ValueMatches(Index: integer; const Search: string): boolean; override;
    procedure SearchKeyPressed(Sender: TFarDialog.TControl; Key, ControlState: DWORD; var Handled: Boolean); override;
    procedure ButtonRescanClicked(Sender: TFarDialog.TButton; var StopProcessing: boolean);
    procedure ButtonEditorClicked(Sender: TFarDialog.TButton; var StopProcessing: boolean);
    function EngineInfo(const Item: IVirusTotalReportItem): string;
    function VirusInfo(const Item: IVirusTotalReportItem): string;
    function ItemInfo(const Item: IVirusTotalReportItem; MaxEngineNameWidth: integer = MaxInt): string;
    procedure DoReloadReport;
    procedure ShowReportInEditor;
    procedure RefreshDialogFromReport;
    property ButtonRescan: TFarDialog.TButton read fButtonRescan;
    property ButtonEditor: TFarDialog.TButton read fButtonEditor;
    property LabelTotals: TFarDialog.TLabel read fLabelTotals;
  public
    constructor Create;
    destructor Destroy; override;
    function Execute: boolean;
    function ValueExists(const Search: string): boolean;
    property FileName: string read fFileName write fFileName;
    property Report: IVirusTotalReport read fReport write fReport;
    property Data: TStream read fData write fData;
    property OnReloadReport: TReloadReportEvent read fOnReloadReport write fOnReloadReport;
  end;

implementation

uses
  uPluginConsts, uMessages;

{ TReportDialog }

procedure TReportDialog.ButtonEditorClicked(Sender: TFarDialog.TButton; var StopProcessing: boolean);
begin
  StopProcessing := True;
  ActiveControl := EditSearch;
  ShowReportInEditor;
end;

procedure TReportDialog.ButtonRescanClicked(Sender: TFarDialog.TButton; var StopProcessing: boolean);
begin
  StopProcessing := True;
  ActiveControl := EditSearch;
  DoReloadReport;
end;

procedure TReportDialog.Clear;
begin
  inherited;
  fButtonRescan := nil;
  fButtonEditor := nil;
  fLabelTotals := nil;
end;

constructor TReportDialog.Create;
begin
  inherited Create(DIALOG_REPORT_GUID);
  fReport := nil;
  Description := GetMsg(MVirusTotalReport);
end;

procedure TReportDialog.CreateControls;
begin
  inherited;
  fButtonRescan := AddButton(0, 0, GetMsg(MRefreshButton));
  fButtonRescan.NoClose := True;
  fButtonRescan.OnClick := ButtonRescanClicked;
  fButtonEditor := AddButton(0, 0, GetMsg(MEditorButton));
  fButtonEditor.NoClose := True;
  fButtonEditor.OnClick := ButtonEditorClicked;
  fLabelTotals := AddLabel(0, 0, 0, GetMsg(MDetections), taCenter);
end;

destructor TReportDialog.Destroy;
begin
  fReport := nil;
  inherited;
end;

procedure TReportDialog.DoReloadReport;
var
  OldReport, NewReport: IVirusTotalReport;
begin
  if Assigned(OnReloadReport) then
  begin
    try
      NewReport := nil;
      OnReloadReport(Self, NewReport);
      if Assigned(NewReport) then
      begin
        OldReport := Self.Report;
        Self.Report := NewReport;
        try
          RefreshDialogFromReport;
        finally
          OldReport := nil;
        end;
      end;
    except
      on EAbort do
        ;
    end;
  end;
end;

function TReportDialog.EngineInfo(const Item: IVirusTotalReportItem): string;
begin
  Result := Item.Engine;
  if (Result <> '') and (Item.EngineVersion <> '') then
    Result := Result + ' ' + Item.EngineVersion;
end;

function TReportDialog.Execute: boolean;
begin
  RefreshDialogFromReport;
  // Enable reload, if possible
  ButtonRescan.Enabled := Assigned(OnReloadReport);
  // Run the dialog
  Result := inherited Execute;
end;

function TReportDialog.ItemInfo(const Item: IVirusTotalReportItem; MaxEngineNameWidth: integer): string;
begin
  Result := Format('%-*.*s  |  %s', [
          MaxEngineNameWidth, MaxEngineNameWidth, EngineInfo(Item)
        , VirusInfo(Item)
      ]);
end;

procedure TReportDialog.RefreshDialogFromReport;
var
  i: integer;
  Item: IVirusTotalReportItem;
  ContentWidth: integer;
  EngineNameWidth, MaxEngineNameWidth: integer;
  DetectionCount: integer;
  ScanDate: TDateTime;
  sScanDate: string;
begin
  ContentWidth := ListValues.Width - 2 {frame} - 5 {separators} ;
  // Find the necessary display info
  MaxEngineNameWidth := 0;
  DetectionCount := 0;
  for i := 0 to Pred(Report.Count) do
  begin
    Item := Report.Items[i];
    EngineNameWidth := Length(EngineInfo(Item));
    if EngineNameWidth > MaxEngineNameWidth then
      MaxEngineNameWidth := EngineNameWidth;
    if Item.Detected then
      Inc(DetectionCount);
  end;
  if MaxEngineNameWidth > (ContentWidth div 2) then
    MaxEngineNameWidth := ContentWidth div 2;
  Report.Sort(
    function(const Left, Right: IVirusTotalReportItem): integer
    begin
      Result := 0;
      // Sort by detection status
      if Result = 0 then
        if Left.Detected then
          if Right.Detected then
            Result := 0
          else
            Result := -1
        else
          if Right.Detected then
            Result := 1
          else
            Result := 0;
      // Sort by engine name
      if Result = 0 then
        Result := AnsiCompareText(Left.Engine, Right.Engine);
    end
  );
  // Create a list of indices which we want
  // Generate the display values
  ListValues.BeginUpdate;
  try
    ListValues.Items.Clear;
    for i := 0 to Pred(Report.Count) do
    begin
      ListValues.Items.Add(ItemInfo(Report.Items[i], MaxEngineNameWidth));
    end;
  finally
    ListValues.EndUpdate;
  end;
  // Generate the totals
  ScanDate := Report.ScanDate;
  if ScanDate > 0 then
    sScanDate := DateTimeToStr(ScanDate)
  else
    sScanDate := GetMsg(MDateUnknown);
  LabelTotals.Text := Format(GetMsg(MDetectionTotals), [DetectionCount, (Report.Count-DetectionCount), Report.Count, DateTimeToStr(ScanDate)]);
end;

procedure TReportDialog.SetSize(Width, Height: integer);
begin
  inherited;
  ResizeListValues(-1);
  AutoPos(LabelTotals);
  LabelTotals.Left := ListValues.Left + 2;
  LabelTotals.Width := ListValues.Width - 4;
  ButtonRescan.Top := ButtonOK.Top;
  ButtonEditor.Top := ButtonOK.Top;
  RefreshDialogFromReport;
end;

procedure TReportDialog.ShowReportInEditor;
const
  EOLN = #13#10;
  SPLITTER = '------------------------------------------------------------';
var
  Content: string;
  ContentUTF8: RawByteString;
  ContentFileName: string;
  ContentStream: TFileStream;
  ScanDate: TDateTime;
  sScanDate: string;
  i: integer;
  Item: IVirusTotalReportItem;
  DetectionCount: integer;
  EngineNameWidth, MaxEngineNameWidth: integer;
  EditorID: TIntPtr;
begin
  DetectionCount := 0;
  MaxEngineNameWidth := 0;
  for i := 0 to Pred(Report.Count) do
  begin
    Item := Report.Items[i];
    EngineNameWidth := Length(EngineInfo(Item));
    if EngineNameWidth > MaxEngineNameWidth then
      MaxEngineNameWidth := EngineNameWidth;
    if Item.Detected then
      Inc(DetectionCount);
  end;
  ScanDate := Report.ScanDate;
  if ScanDate > 0 then
    sScanDate := DateTimeToStr(ScanDate)
  else
    sScanDate := GetMsg(MEditorScanDateUnknown);
  Content := ''
    + GetMsg(MTitle) + EOLN
    + SPLITTER + EOLN
    + GetMsg(MEditorReportOnFile) + FileName + EOLN
    + GetMsg(MEditorScanDate) + sScanDate + EOLN
    + GetMsg(MEditorScanID) + Report.ScanID + EOLN
    + GetMsg(MEditorResourceID) + Report.ResourceID + EOLN
    + GetMsg(MEditorPermalink) + Report.Permalink + EOLN
    + Format(GetMsg(MEditorDetectionCount), [DetectionCount, Report.Count]) + EOLN
    + SPLITTER + EOLN
    ;
  for i := 0 to Pred(Report.Count) do
  begin
    Item := Report.Items[i];
    if Item.Detected then
    begin
      Content := Content
        + Format('%-*.*s', [MaxEngineNameWidth, MaxEngineNameWidth, EngineInfo(Item)])
        + ' | '
        + VirusInfo(Item)
        + EOLN
        ;
    end;
  end;
  Content := Content + SPLITTER + EOLN;
  for i := 0 to Pred(Report.Count) do
  begin
    Item := Report.Items[i];
    if not Item.Detected then
    begin
      Content := Content + EngineInfo(Item) + EOLN;
    end;
  end;
  ContentUTF8 := #$ef#$bb#$bf + UTF8Encode(Content);
  ContentFileName := TempFileName;
  ContentStream := TFileStream.Create(ContentFileName, fmCreate);
  try
    ContentStream.WriteBuffer(ContentUTF8[1], Length(ContentUTF8));
  finally
    FreeAndNil(ContentStream);
  end;
  TFarEditor.Show(EditorID, ContentFileName, GetMsg(MTitle), 0, 0, -1, -1, EF_NONMODAL or EF_IMMEDIATERETURN or EF_DELETEONLYFILEONCLOSE, 0, 1, CP_UTF8);
end;

procedure TReportDialog.SearchKeyPressed(Sender: TFarDialog.TControl; Key, ControlState: DWORD; var Handled: Boolean);
const
  ControlKeyMask = {$IFDEF FAR3_UP} LEFT_ALT_PRESSED or RIGHT_ALT_PRESSED or LEFT_CTRL_PRESSED or RIGHT_CTRL_PRESSED or SHIFT_PRESSED {$ELSE} KEY_CTRLMASK {$ENDIF} ;
{$IFDEF FAR3_UP}
const
  KEY_F4 = VK_F4;
  KEY_F5 = VK_F5;
{$ENDIF}
begin
  if (not Handled) and ((ControlState and ControlKeyMask) = 0) and (Key = KEY_F5) and ButtonRescan.Enabled then
  begin
    Handled := True;
    ActiveControl := EditSearch;
    DoReloadReport;
    Exit;
  end;
  if (not Handled) and ((ControlState and ControlKeyMask) = 0) and (Key = KEY_F4) and ButtonRescan.Enabled then
  begin
    Handled := True;
    ActiveControl := EditSearch;
    ShowReportInEditor;
    Exit;
  end;
  inherited;
end;

function TReportDialog.ValueExists(const Search: string): boolean;
var
  i: integer;
begin
  Result := False;
  for i := 0 to Pred(Report.Count) do
  begin
    if ValueMatches(i, Search) then
    begin
      Result := True;
      Break;
    end;
  end;
end;

function TReportDialog.ValueMatches(Index: integer; const Search: string): boolean;
var
  Item: IVirusTotalReportItem;
begin
  if Search = '' then
  begin
    Result := True;
    Exit;
  end;
  Item := Report.Items[Index];
  Result := False
    or ContainsText(Item.Engine, Search)
    or ContainsText(Item.Name, Search)
    ;
end;

function TReportDialog.VirusInfo(const Item: IVirusTotalReportItem): string;
begin
  Result := Item.Name;
end;

end.
